@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_rsa.c,v 1.48 2022/08/31 20:49:37 tb Exp $ */
+/* $OpenBSD: ssl_rsa.c,v 1.50 2023/07/08 16:40:13 beck Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -64,7 +64,7 @@
 #include <openssl/pem.h>
 #include <openssl/x509.h>
 
-#include "ssl_locl.h"
+#include "ssl_local.h"
 
 static int ssl_get_password_cb_and_arg(SSL_CTX *ctx, SSL *ssl,
     pem_password_cb **passwd_cb, void **passwd_arg);
@@ -83,6 +83,7 @@ SSL_use_certificate(SSL *ssl, X509 *x)
 	}
 	return ssl_set_cert(NULL, ssl, x);
 }
+LSSL_ALIAS(SSL_use_certificate);
 
 int
 SSL_use_certificate_file(SSL *ssl, const char *file, int type)
@@ -126,6 +127,7 @@ SSL_use_certificate_file(SSL *ssl, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_certificate_file);
 
 int
 SSL_use_certificate_ASN1(SSL *ssl, const unsigned char *d, int len)
@@ -143,6 +145,7 @@ SSL_use_certificate_ASN1(SSL *ssl, const unsigned char *d, int len)
 	X509_free(x);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_certificate_ASN1);
 
 int
 SSL_use_RSAPrivateKey(SSL *ssl, RSA *rsa)
@@ -166,6 +169,7 @@ SSL_use_RSAPrivateKey(SSL *ssl, RSA *rsa)
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_RSAPrivateKey);
 
 static int
 ssl_set_pkey(SSL_CTX *ctx, SSL *ssl, EVP_PKEY *pkey)
@@ -259,6 +263,7 @@ SSL_use_RSAPrivateKey_file(SSL *ssl, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_RSAPrivateKey_file);
 
 int
 SSL_use_RSAPrivateKey_ASN1(SSL *ssl, const unsigned char *d, long len)
@@ -275,6 +280,7 @@ SSL_use_RSAPrivateKey_ASN1(SSL *ssl, const unsigned char *d, long len)
 	RSA_free(rsa);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_RSAPrivateKey_ASN1);
 
 int
 SSL_use_PrivateKey(SSL *ssl, EVP_PKEY *pkey)
@@ -288,6 +294,7 @@ SSL_use_PrivateKey(SSL *ssl, EVP_PKEY *pkey)
 	ret = ssl_set_pkey(NULL, ssl, pkey);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_PrivateKey);
 
 int
 SSL_use_PrivateKey_file(SSL *ssl, const char *file, int type)
@@ -328,6 +335,7 @@ SSL_use_PrivateKey_file(SSL *ssl, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_PrivateKey_file);
 
 int
 SSL_use_PrivateKey_ASN1(int type, SSL *ssl, const unsigned char *d, long len)
@@ -344,6 +352,7 @@ SSL_use_PrivateKey_ASN1(int type, SSL *ssl, const unsigned char *d, long len)
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
+LSSL_ALIAS(SSL_use_PrivateKey_ASN1);
 
 int
 SSL_CTX_use_certificate(SSL_CTX *ctx, X509 *x)
@@ -354,6 +363,7 @@ SSL_CTX_use_certificate(SSL_CTX *ctx, X509 *x)
 	}
 	return ssl_set_cert(ctx, NULL, x);
 }
+LSSL_ALIAS(SSL_CTX_use_certificate);
 
 static int
 ssl_get_password_cb_and_arg(SSL_CTX *ctx, SSL *ssl,
@@ -475,6 +485,7 @@ SSL_CTX_use_certificate_file(SSL_CTX *ctx, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_certificate_file);
 
 int
 SSL_CTX_use_certificate_ASN1(SSL_CTX *ctx, int len, const unsigned char *d)
@@ -492,6 +503,7 @@ SSL_CTX_use_certificate_ASN1(SSL_CTX *ctx, int len, const unsigned char *d)
 	X509_free(x);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_certificate_ASN1);
 
 int
 SSL_CTX_use_RSAPrivateKey(SSL_CTX *ctx, RSA *rsa)
@@ -515,6 +527,7 @@ SSL_CTX_use_RSAPrivateKey(SSL_CTX *ctx, RSA *rsa)
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_RSAPrivateKey);
 
 int
 SSL_CTX_use_RSAPrivateKey_file(SSL_CTX *ctx, const char *file, int type)
@@ -555,6 +568,7 @@ SSL_CTX_use_RSAPrivateKey_file(SSL_CTX *ctx, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_RSAPrivateKey_file);
 
 int
 SSL_CTX_use_RSAPrivateKey_ASN1(SSL_CTX *ctx, const unsigned char *d, long len)
@@ -571,6 +585,7 @@ SSL_CTX_use_RSAPrivateKey_ASN1(SSL_CTX *ctx, const unsigned char *d, long len)
 	RSA_free(rsa);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_RSAPrivateKey_ASN1);
 
 int
 SSL_CTX_use_PrivateKey(SSL_CTX *ctx, EVP_PKEY *pkey)
@@ -581,6 +596,7 @@ SSL_CTX_use_PrivateKey(SSL_CTX *ctx, EVP_PKEY *pkey)
 	}
 	return ssl_set_pkey(ctx, NULL, pkey);
 }
+LSSL_ALIAS(SSL_CTX_use_PrivateKey);
 
 int
 SSL_CTX_use_PrivateKey_file(SSL_CTX *ctx, const char *file, int type)
@@ -621,6 +637,7 @@ SSL_CTX_use_PrivateKey_file(SSL_CTX *ctx, const char *file, int type)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_PrivateKey_file);
 
 int
 SSL_CTX_use_PrivateKey_ASN1(int type, SSL_CTX *ctx, const unsigned char *d,
@@ -638,6 +655,7 @@ SSL_CTX_use_PrivateKey_ASN1(int type, SSL_CTX *ctx, const unsigned char *d,
 	EVP_PKEY_free(pkey);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_PrivateKey_ASN1);
 
 
 /*
@@ -721,12 +739,14 @@ SSL_CTX_use_certificate_chain_file(SSL_CTX *ctx, const char *file)
 {
 	return ssl_use_certificate_chain_file(ctx, NULL, file);
 }
+LSSL_ALIAS(SSL_CTX_use_certificate_chain_file);
 
 int
 SSL_use_certificate_chain_file(SSL *ssl, const char *file)
 {
 	return ssl_use_certificate_chain_file(NULL, ssl, file);
 }
+LSSL_ALIAS(SSL_use_certificate_chain_file);
 
 int
 SSL_CTX_use_certificate_chain_mem(SSL_CTX *ctx, void *buf, int len)
@@ -746,3 +766,4 @@ SSL_CTX_use_certificate_chain_mem(SSL_CTX *ctx, void *buf, int len)
 	BIO_free(in);
 	return (ret);
 }
+LSSL_ALIAS(SSL_CTX_use_certificate_chain_mem);
